==> openGion6.0.0.3_dev.zip: uap/webapps/gf/src/org/opengion/fukurou/util/Attributes.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.util.Set;
import java.util.Map;
import java.util.HashMap;
import java.util.Arrays;
import java.util.Locale;

/**
 * Attributes.java は、String 型キーにString型値を Map するクラスです。
 *
 * HTMLのPOST/GET等の受け渡しや、String型の引数が多い場合に効果があります。
 * 特に、getAttributes( String[] param ) による属性リスト作成は、
 * HTMLタグの属性定義を行う上で，非常に便利に利用できます。
 *
 * この実装は同期化されません。
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public final class Attributes {
	private final Map<String,String> attri ;

	/**
	 * デフォルトコンストラクター
	 *
	 */
	public Attributes() {
		attri = new HashMap<String,String>();
	}

	/**
	 * Attributesオブジェクト を与えて新しく作成するコンストラクター
	 *
	 * @param att Attributesオブジェクト
	 */
	public Attributes( final Attributes att ) {
		attri = new HashMap<String,String>( att.attri );
	}

	/**
	 * マップからマッピングをすべて削除します 。
	 *
	 */
	public void clear()  {
		attri.clear() ;
	}

	/**
	 * マップが指定のキーをマップする値を返します。
	 * マップがこのキーのマッピングを保持していない場合は null
	 * を返します。戻り値の null は、マップがキーのマッピングを
	 * 保持していないことを示すとはかぎりません。つまり、マップが
	 * 明示的にキーを null にマップすることもあります。
	 *
	 * @param    key 関連付けられた値が返されるキー(大文字小文字は同値)
	 *
	 * @return   マップが、指定されたキーにマッピングしている値。
	 *           このキーに対するマッピングがマップにない場合は null
	 */
	public String get( final String key ) {
		return attri.get( key.toLowerCase( Locale.JAPAN ) ) ;
	}

	/**
	 * 指定された値と指定されたキーをこのマップに関連付けます
	 * 指定されたキーに、null を関連付けることはできません。
	 * (もちろん、""：ゼロストリング は登録できます。)
	 * なぜなら、getAttribute( String[] keys ) 等で値が null の
	 * キーは、取得できない為です。
	 * また、すでに何らかの値がセットされている所に､null をセットした
	 * 場合は､前の値をなにも変更しません。
	 * 通常、値をクリアしたい場合は､ remove( String key ) を利用してください。
	 *
	 * @param    key 指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーに関連付けられる値
	 */
	public void set( final String key,final String value ) {
		if( value != null ) {
			attri.put( key.toLowerCase( Locale.JAPAN ),value ) ;
		}
	}

	/**
	 * 指定された値と指定されたキーをこのマップに関連付けます
	 * set( String key,String value ) との違いは､value が null
	 * の場合に、def を代わりにセットすることです。
	 * ただし、value が null で、def も null の場合は､
	 * なにもセットされません。
	 *
	 * @param    key 指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーに関連付けられる値
	 * @param    def value が null の場合にキーに関連付けられる値
	 */
	public void set( final String key,final String value,final String def ) {
		if( value != null )    { attri.put( key.toLowerCase( Locale.JAPAN ),value ) ; }
		else if( def != null ) { attri.put( key.toLowerCase( Locale.JAPAN ),def )   ; }
	}

	/**
	 * 指定された値と指定されたキーをこのマップに追加します
	 *
	 * マップ自身のキーは、ユニークである為、既存の値に対して、
	 * 新しく値を追加します。
	 * 追加する方法は、値の文字列の結合です。このメソッドでは、
	 * デフォルトのスペースで結合します。
	 *
	 * 値が null または、すでにそのキーに同一の値が関連付けられている場合は、
	 * 何もしません。
	 *
	 * @param    key   指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーの値に、追加される値
	 */
	public void add( final String key,final String value ) {
		add( key,value," " ) ;
	}

	/**
	 * 指定された値と指定されたキーをこのマップに追加します
	 *
	 * マップ自身のキーは、ユニークである為、既存の値に対して、
	 * 新しく値を追加します。
	 * 追加する方法は、値の文字列の結合です。このメソッドでは、
	 * 引数 sepa で文字列を結合します。
	 *
	 * 値が null または、sepa が null または、すでにそのキーに
	 * 同一の値が関連付けられている場合は、何もしません。
	 *
	 * @param    key   指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーの値に、追加される値
	 * @param    sepa  値を連結するときの文字列
	 */
	public void add( final String key,final String value,final String sepa ) {
		if( value != null && sepa != null ) {
			String lkey = key.toLowerCase( Locale.JAPAN );
			String temp = attri.get( lkey );
			if( temp != null ) {
				temp = temp.trim();
				if( temp.indexOf( value ) < 0 ||			// 存在しない または、
					( ! temp.equals( value ) &&				// 一致しない
					  ! temp.startsWith( value + sepa ) &&	// 先頭にない
					  ! temp.endsWith( sepa + value ) &&	// 最終にない
					  temp.indexOf( sepa + value + sepa ) < 0 ) ) {	// 途中にない
						if( temp.endsWith( sepa ) ) {
							attri.put( lkey,temp + value );
						}
						else {
							attri.put( lkey,temp + sepa + value );
						}
				}
			}
			else {
				attri.put( lkey,value );
			}
		}
	}

	/**
	 * Attributes 属性を、既存の属性に追加します。
	 * すでに同一キーの属性が存在している場合は，上書きで
	 * 置き換えます。
	 * 引数 att が null の場合は，何もしません。
	 *
	 * @param att Attributes属性
	 */
	public void addAttributes( final Attributes att ) {
		if( att != null ) {
			String[] keys = att.getKeyArray();
			for( int i=0; i<keys.length; i++ ) {
				set( keys[i],att.get( keys[i] ) );
			}
		}
	}

	/**
	 * このキーにマッピングがある場合に、そのマッピングをマップから削除します
	 *
	 * @param    key マッピングがマップから削除されるキー(大文字小文字は同値)
	 *
	 * @return   このキーにマッピングがある場合に、そのマッピングをマップから削除します
	 *           指定されたキーに関連した以前の値。key にマッピングがなかった場合は null。
	 */
	public String remove( final String key ) {
		return attri.remove( key.toLowerCase( Locale.JAPAN ) );
	}

	/**
	 * マップ内のキーと値のマッピングの数を返します。
	 *
	 * @return   インタフェース Map 内の size
	 *
	 */
	public int size() {
		return attri.size() ;
	}

	/**
	 * マップに含まれているキーの配列を返します。
	 * ここでは、キーの配列はソートして返します。
	 *
	 * @return   マップに含まれているキーの配列
	 *
	 */
	public String[] getKeyArray() {
		Set<String> keyset = attri.keySet();
		String[] rtn = keyset.toArray( new String[keyset.size()] ) ;
		Arrays.sort( rtn );
		return rtn ;
	}

	/**
	 * マップに含まれているキーと属性のペアを タグの属性リストの形式で返します。
	 * key1="value1" key2="value2" key3="value3" .... の形式で、value が null の
	 * 場合は，key そのもののペアを出力しません。
	 * value が空文字列 "" の場合は，key="" で出力します。
	 *
	 * 引数には，key として出力したい値を配列文字列で渡します。
	 * これは、拡張性に乏しい(すべて出せば，属性項目の追加に対応できる。)
	 * 方法ですが、タグ毎に異なる属性のみを管理するには，厳格に出力
	 * タグ属性を定義したいという思いから，導入しました。
	 *
	 * @param    keys 指定 key の配列文字列(大文字小文字は同値)
	 *
	 * @return   キーと属性のペアをタグの属性リストの形式で返します
	 *
	 */
	public String getAttribute( final String[] keys ) {
		StringBuilder buf = new StringBuilder( 200 );

		for( int i=0; i<keys.length; i++ ) {
			String value = get( keys[i].toLowerCase( Locale.JAPAN ) );
			if( value != null ) {
				buf.append( keys[i] ).append("=");
				buf.append("\"").append( value ).append("\" ");
			}
		}
		return buf.toString();
	}

	/**
	 * マップに含まれているキーと属性のペアを タグの属性リストの形式ですべて返します。
	 * なお、value が null の場合は，key そのもののペアを出力しません。
	 * value が空文字列 "" の場合は，key="" で出力します。
	 *
	 * @return   キーと属性のペアをタグの属性リストの形式で返します
	 *
	 */
	public String getAttribute() {
		return getAttribute( getKeyArray() );
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return	オブジェクトの文字列表現
	 */
	@Override
	public String toString() {
		return getAttribute() ;
	}
}

==> openGion6.1.0.0_dev.zip: uap/webapps/gf/src/org/opengion/fukurou/util/Attributes.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.util;

import java.util.Map;
import java.util.HashMap;
import java.util.Locale;

import static org.opengion.fukurou.util.HybsConst.BUFFER_MIDDLE;	// 6.1.0.0 (2014/12/26) refactoring

/**
 * Attributes.java は、String 型キーにString型値を Map するクラスです。
 *
 * HTMLのPOST/GET等の受け渡しや、String型の引数が多い場合に効果があります。
 * 特に、getAttributes( String[] param ) による属性リスト作成は、
 * HTMLタグの属性定義を行う上で，非常に便利に利用できます。
 *
 * この実装は同期化されません。
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public final class Attributes {
	private final Map<String,String> attri ;

	/**
	 * デフォルトコンストラクター
	 *
	 */
	public Attributes() {
		attri = new HashMap<String,String>();
	}

	/**
	 * Attributesオブジェクト を与えて新しく作成するコンストラクター
	 *
	 * @param att Attributesオブジェクト
	 */
	public Attributes( final Attributes att ) {
		attri = new HashMap<String,String>( att.attri );
	}

	/**
	 * マップからマッピングをすべて削除します 。
	 *
	 */
	public void clear()  {
		attri.clear() ;
	}

	/**
	 * マップが指定のキーをマップする値を返します。
	 * マップがこのキーのマッピングを保持していない場合は null
	 * を返します。戻り値の null は、マップがキーのマッピングを
	 * 保持していないことを示すとはかぎりません。つまり、マップが
	 * 明示的にキーを null にマップすることもあります。
	 *
	 * @param    key 関連付けられた値が返されるキー(大文字小文字は同値)
	 *
	 * @return   マップが、指定されたキーにマッピングしている値。
	 *           このキーに対するマッピングがマップにない場合は null
	 */
	public String get( final String key ) {
		return attri.get( key.toLowerCase( Locale.JAPAN ) ) ;
	}

	/**
	 * 指定された値と指定されたキーをこのマップに関連付けます
	 * 指定されたキーに、null を関連付けることはできません。
	 * (もちろん、""：ゼロストリング は登録できます。)
	 * なぜなら、getAttribute( String[] keys ) 等で値が null の
	 * キーは、取得できない為です。
	 * また、すでに何らかの値がセットされている所に､null をセットした
	 * 場合は､前の値をなにも変更しません。
	 * 通常、値をクリアしたい場合は､ remove( String key ) を利用してください。
	 *
	 * @param    key 指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーに関連付けられる値
	 */
	public void set( final String key,final String value ) {
		if( value != null ) {
			attri.put( key.toLowerCase( Locale.JAPAN ),value ) ;
		}
	}

	/**
	 * 指定された値と指定されたキーをこのマップに関連付けます
	 * set( String key,String value ) との違いは､value が null
	 * の場合に、def を代わりにセットすることです。
	 * ただし、value が null で、def も null の場合は､
	 * なにもセットされません。
	 *
	 * @param    key 指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーに関連付けられる値
	 * @param    def value が null の場合にキーに関連付けられる値
	 */
	public void set( final String key,final String value,final String def ) {
		if( value != null )    { attri.put( key.toLowerCase( Locale.JAPAN ),value ) ; }
		else if( def != null ) { attri.put( key.toLowerCase( Locale.JAPAN ),def )   ; }
	}

	/**
	 * 指定された値と指定されたキーをこのマップに追加します
	 *
	 * マップ自身のキーは、ユニークである為、既存の値に対して、
	 * 新しく値を追加します。
	 * 追加する方法は、値の文字列の結合です。このメソッドでは、
	 * デフォルトのスペースで結合します。
	 *
	 * 値が null または、すでにそのキーに同一の値が関連付けられている場合は、
	 * 何もしません。
	 *
	 * @param    key   指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーの値に、追加される値
	 */
	public void add( final String key,final String value ) {
		add( key,value," " ) ;
	}

	/**
	 * 指定された値と指定されたキーをこのマップに追加します
	 *
	 * マップ自身のキーは、ユニークである為、既存の値に対して、
	 * 新しく値を追加します。
	 * 追加する方法は、値の文字列の結合です。このメソッドでは、
	 * 引数 sepa で文字列を結合します。
	 *
	 * 値が null または、sepa が null または、すでにそのキーに
	 * 同一の値が関連付けられている場合は、何もしません。
	 *
	 * @param    key   指定される値が関連付けられるキー(大文字小文字は同値)
	 * @param    value 指定されるキーの値に、追加される値
	 * @param    sepa  値を連結するときの文字列
	 */
	public void add( final String key,final String value,final String sepa ) {
		if( value != null && sepa != null ) {
			final String lkey = key.toLowerCase( Locale.JAPAN );
			String temp = attri.get( lkey );
			if( temp != null ) {
				temp = temp.trim();
				if( temp.indexOf( value ) < 0 ||			// 存在しない または、
					( ! temp.equals( value ) &&				// 一致しない
					  ! temp.startsWith( value + sepa ) &&	// 先頭にない
					  ! temp.endsWith( sepa + value ) &&	// 最終にない
					  temp.indexOf( sepa + value + sepa ) < 0 ) ) {	// 途中にない
						if( temp.endsWith( sepa ) ) {
							attri.put( lkey,temp + value );
						}
						else {
							attri.put( lkey,temp + sepa + value );
						}
				}
			}
			else {
				attri.put( lkey,value );
			}
		}
	}

	/**
	 * Attributes 属性を、既存の属性に追加します。
	 * すでに同一キーの属性が存在している場合は，上書きで
	 * 置き換えます。
	 * 引数 att が null の場合は，何もしません。
	 *
	 * @og.rev 6.0.4.0 (2014/11/28) 内部処理見直し。キーのソートを行いません。
	 *
	 * @param att Attributes属性
	 */
	public void addAttributes( final Attributes att ) {
		if( att != null ) {
//			String[] keys = att.getKeyArray();
//			for( int i=0; i<keys.length; i++ ) {
//				set( keys[i],att.get( keys[i] ) );
//			}
			attri.putAll( att.attri );
		}
	}

	/**
	 * このキーにマッピングがある場合に、そのマッピングをマップから削除します。
	 *
	 * @param    key マッピングがマップから削除されるキー(大文字小文字は同値)
	 *
	 * @return   このキーにマッピングがある場合に、そのマッピングをマップから削除します
	 *           指定されたキーに関連した以前の値。key にマッピングがなかった場合は null。
	 */
	public String remove( final String key ) {
		return attri.remove( key.toLowerCase( Locale.JAPAN ) );
	}

	/**
	 * マップ内のキーと値のマッピングの数を返します。
	 *
	 * @return   インタフェース Map 内の size
	 */
	public int size() {
		return attri.size() ;
	}

//	/**
//	 * マップに含まれているキーの配列を返します。
//	 * ここでは、キーの配列はソートして返します。
//	 *
//	 * @og.rev 6.0.4.0 (2014/11/28) 他に使っていないので、廃止します。
//	 *
//	 * @return   マップに含まれているキーの配列
//	 */
//	public String[] getKeyArray() {
//		Set<String> keyset = attri.keySet();
//		String[] rtn = keyset.toArray( new String[keyset.size()] ) ;
//		Arrays.sort( rtn );
//		return rtn ;
//	}

	/**
	 * マップに含まれているキーと属性のペアを タグの属性リストの形式で返します。
	 * key1="value1" key2="value2" key3="value3" .... の形式で、value が null の
	 * 場合は，key そのもののペアを出力しません。
	 * value が空文字列 "" の場合は，key="" で出力します。
	 *
	 * 引数には，key として出力したい値を配列文字列で渡します。
	 * これは、拡張性に乏しい(すべて出せば，属性項目の追加に対応できる。)
	 * 方法ですが、タグ毎に異なる属性のみを管理するには，厳格に出力
	 * タグ属性を定義したいという思いから，導入しました。
	 *
	 * @og.rev 6.0.4.0 (2014/11/28) 内部処理見直し。値の取得は、Mapから直接取得する。
	 *
	 * @param    keys 指定 key の文字列配列(可変長引数)(大文字小文字は同値)
	 *
	 * @return   キーと属性のペアをタグの属性リストの形式で返します
	 */
//	public String getAttribute( final String[] keys ) {
	public String getAttribute( final String... keys ) {
		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );

		for( int i=0; i<keys.length; i++ ) {
//			String value = get( keys[i].toLowerCase( Locale.JAPAN ) );
			final String value = attri.get( keys[i].toLowerCase( Locale.JAPAN ) );	// 6.0.4.0 (2014/11/28) Mapから直接取得する。
			if( value != null ) {
				// 6.0.2.5 (2014/10/31) char を append する。
				buf.append( keys[i] ).append("=\"").append( value ).append("\" ");
			}
		}
		return buf.toString();
	}

	/**
	 * マップに含まれているキーと属性のペアを タグの属性リストの形式ですべて返します。
	 * なお、value が null の場合は，key そのもののペアを出力しません。
	 * value が空文字列 "" の場合は，key="" で出力します。
	 *
	 * @og.rev 6.0.4.0 (2014/11/28) 内部処理見直し。値の取得は、Mapから直接取得する。
	 *
	 * @return   キーと属性のペアをタグの属性リストの形式で返します
	 */
	public String getAttribute() {
//		return getAttribute( getKeyArray() );
		final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );

		// 内部に持っているすべてのデータを出力する。
		// よって、toLowerCase も、null チェックも不要。
		for( final Map.Entry<String,String> entry : attri.entrySet() ) {
			buf.append( entry.getKey() ).append("=\"").append( entry.getValue() ).append("\" ");
		}

		return buf.toString();
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return	オブジェクトの文字列表現
	 */
	@Override
	public String toString() {
		return getAttribute() ;
	}
}
